﻿/*
 * Przykład ze strony pod adresem:
 * http://www.devexpress.com/Help/Content.aspx?help=XPO&document=CustomDocument4060.htm
 * http://www.devexpress.com/Support/Center/KB/p/K18051.aspx
 * 
 * Przed odpaleniem przykładu trzeba się upewnić,
 * że w SQL Server jest dostępna baza Northwind; do pobrania z adresu:
 * http://www.microsoft.com/downloads/details.aspx?FamilyID=06616212-0356-46A0-8DA2-EEBC53A68034&displaylang=en
 * 
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using DevExpress.Xpo;
using System.Configuration;

namespace LinqToXpoExample
{
    [Persistent( "Customers" )]
    public class Customer : XPLiteObject
    {
        public Customer( Session session ) : base( session ) { }

        [Key]
        public string CustomerID;

        public string CompanyName;
        public string ContactTitle;
        public string ContactName;
        public string Country;
        public string City;

        [Association( "CustomerOrders", typeof( Order ) )]
        public XPCollection<Order> Orders
        {
            get
            {
                return GetCollection<Order>( "Orders" );
            }
        }
    }

    [Persistent( "Orders" )]
    public class Order : XPLiteObject
    {
        public Order( Session session ) : base( session ) { }

        [Key]
        public int OrderID;
        public DateTime ShippedDate;

        [Persistent( "CustomerID" ), Association( "CustomerOrders" )]
        public Customer Customer;

        [Persistent( "EmployeeID" ), Association( "EmployeeOrders" )]
        public Employee Employee;

        public decimal Freight;
    }

    [Persistent( "Employees" )]
    public class Employee : XPLiteObject
    {
        public Employee( Session session ) : base( session ) { }

        [Key]
        public int EmployeeID;
        public string FirstName;
        public string LastName;

        [Association( "EmployeeOrders" )]
        public XPCollection<Order> Orders
        {
            get
            {
                return GetCollection<Order>( "Orders" );
            }
        }
    }
    class Program
    {
        static void Main( string[] args )
        {
            XpoDefault.ConnectionString = ConfigurationManager.AppSettings["ConnectionString"];

            XPQuery<Customer> customers = new XPQuery<Customer>( Session.DefaultSession );
            XPQuery<Order> orders = new XPQuery<Order>( Session.DefaultSession );
            XPQuery<Employee> employees = new XPQuery<Employee>( Session.DefaultSession );

            // simple Select with Where and OrderBy clauses
            var list = from c in customers
                       where ( c.Country == "Germany" && c.ContactTitle == "Sales Representative" )
                       orderby c.ContactName
                       select c;
            foreach ( Customer cust in list )
                Console.WriteLine( string.Format( "{0}\t{1}\t{2}", cust.ContactName, cust.Country, cust.ContactTitle ) );

            // Select Top 5 objects
            var list1 = ( from o in orders
                          orderby o.ShippedDate descending
                          select o ).Take( 5 );
            foreach ( Order order in list1 )
                Console.WriteLine( string.Format( "{0}\t{1}\t{2}", order.OrderID, order.ShippedDate, order.Customer.CompanyName ) );

            // Group Join customers with an aggregation on their Orders
            var list2 = from c in customers
                        join o in orders on c equals o.Customer into oo
                        where oo.Count() >= 1
                        select new { c.CompanyName, OrderCount = oo.Count() };
            foreach ( var item in list2 )
                Console.WriteLine( string.Format( "{0}\t{1}", item.CompanyName, item.OrderCount ) );

            // an example of aggregated functions (Count and Average)
            var list3 = from o in orders
                        select o;
            int count = list3.Count();
            Console.WriteLine( string.Format( "Orders Row Count: {0}", count ) );

            decimal avg = list3.Average( x => x.Freight );
            Console.WriteLine( string.Format( "Orders Average Freight: {0:c2}", avg ) );

            // Select with Group By
            var list4 = from c in customers
                        group c by c.ContactTitle into cc
                        where cc.Count() >= 1
                        select new { Title = cc.Key, Count = cc.Count() };
            foreach ( var item in list4 )
                Console.WriteLine( string.Format( "{0}\t{1}", item.Title, item.Count ) );

            // Any method
            bool result = customers.Any( c => c.Country == "Spain" );
            Console.WriteLine( string.Format( "Is there any customer from Spain? {0}", result ? "Yes" : "No" ) );

            result = customers.Any( c => c.Country == "Monaco" );
            Console.WriteLine( string.Format( "Is there any customer from Monaco? {0}", result ? "Yes" : "No" ) );
        }
    }
}
